<?php
namespace Controller;

use Zeedhi\DevOps\Util\Environment;
use Zeedhi\DevOps\Util\Utilities;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Request\DataSet as RequestDataset;
use Zeedhi\Framework\DTO\Request\Filter as RequestFilter;
use \Service\Branch as BranchService;

class Branch extends \Generated\Controller\Branch {
    /** @var BranchService  */
    protected $branchService;
    /** @var Environment*/
    private $environment;

    /**
     * @param Manager $dataSourceManager
     * @param Environment $environment
     * @param BranchService $branchService
     */
    public function __construct(Manager $dataSourceManager, Environment $environment, BranchService $branchService) {
        parent::__construct($dataSourceManager);
        $this->environment = $environment;
        $this->branchService = $branchService;
    }

    /**
     * @param RequestDataset $request
     * @param DTO\Response $response
     * @param bool $showMessage
     */
    public function save(DTO\Request $request, DTO\Response $response, $showMessage = true) {
        $dataSet     = $request->getDataSet();
        $rows = $dataSet->getRows();
        $showMessage = $this->verifyShowMessage($dataSet);
        $organizationId = $this->environment->getOrganizationId();
        foreach ($rows as &$row) {
            $row['ORGANIZATION_ID'] = $organizationId;
        }
        $dataSet = new DataSet($dataSet->getDataSourceName(),$rows);
        $request = new RequestDataset($dataSet,$request->getMethod(),$request->getRoutePath(),$request->getUserId());
        parent::save($request, $response, $showMessage);
    }

    /**
     * @param DTO\Request $request
     * @param DTO\Response $response
     * @param bool $showMessage
     */
    public function delete(DTO\Request $request, DTO\Response $response, $showMessage = true) {
        $dataSet     = $request->getDataSet();
        $showMessage = $this->verifyShowMessage($dataSet);
        parent::delete($request, $response, $showMessage);
    }

    public function find(DTO\Request\Filter $request, DTO\Response $response){
        $request->getFilterCriteria()->addCondition("ORGANIZATION_ID", "=", $this->environment->getCurrentOrganizationId());
        parent::find($request, $response);
    }

    /**
     * @param $dataSet
     * @return bool
     */
    private function verifyShowMessage($dataSet) {
        $showMessage = true;

        foreach ($dataSet->getRows() as $row) {
            if(isset($row['isMessage']) && $row['isMessage'] === false) {
                $showMessage = false;
                break;
            }
        }

        return $showMessage;
    }

    /**
     * @param RequestFilter $request
     * @param Response $response
     */
    public function findBranchesBySubRelease(RequestFilter $request, Response $response)
    {
        $conditions = $request->getFilterCriteria()->getConditions();
        $subReleaseId = empty($conditions[0]) ? 0 : $conditions[0]['value'];
        array_shift($conditions);
        $filter = Utilities::createQueryFiltered($conditions);
        try {
            $branches = $this->branchService->findBranchesBySubRelease($subReleaseId, $filter);
            $response->addDataSet(new DataSet('branches', $branches));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }

    /**
     * @param RequestFilter $request
     * @param Response $response
     */
    public function findBranchByNameAndProduct(RequestFilter $request, Response $response)
    {
        $conditions   = $request->getFilterCriteria()->getConditions();
        $branchName   = Utilities::getValueInFilterDataSourceByName($conditions, 'BRANCH_NAME');
        $productId    = Utilities::getValueInFilterDataSourceByName($conditions, 'PRODUCT_ID');
        $subreleaseId = Utilities::getValueInFilterDataSourceByName($conditions, 'SUBRELEASE_ID');
        try {
            $branch = $this->branchService->findBranchByNameAndProduct($branchName, $productId, $subreleaseId);
            $response->addDataSet(new DataSet('branch', $branch));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }
}

